@@ -50,7 +50,6 @@ static void basep_pos_class_init (BasePPosClass *klass);
 static void basep_pos_instance_init (BasePPos *pos);
 
 /* Forward declare some static functions for use in the class init */
-static void basep_widget_move_focus_out (BasePWidget *basep);
 static void basep_widget_mode_change (BasePWidget *basep, BasePMode mode);
 static void basep_widget_state_change (BasePWidget *basep, BasePState state);
 static void basep_widget_real_screen_change (BasePWidget *basep, int screen);
@@ -60,9 +59,10 @@ static void basep_widget_realize (GtkWidget *w);
 static void basep_widget_map (GtkWidget *w);
 static gboolean basep_enter_notify (GtkWidget *widget, GdkEventCrossing *event);
 static gboolean basep_leave_notify (GtkWidget *widget, GdkEventCrossing *event);
-static gboolean basep_key_press (GtkWidget *widget, GdkEventKey *event);
 static void basep_style_set (GtkWidget *widget, GtkStyle *previous_style);
 static void basep_widget_destroy (GtkObject *o);
+static int  basep_widget_focus_in_event (GtkWidget     *widget,
+					 GdkEventFocus *event);
 
 static void basep_widget_get_property (GObject *object, guint prop_id, GValue *value, GParamSpec *pspec);
 static void basep_widget_set_property (GObject *object, guint prop_id, const GValue *value, GParamSpec *pspec);
@@ -124,14 +124,12 @@ static guint basep_widget_signals[WIDGET_LAST_SIGNAL] = { 0 };
 static void
 basep_widget_class_init (BasePWidgetClass *klass)
 {
-	GObjectClass *object_class = G_OBJECT_CLASS (klass);
-	GtkObjectClass *gtk_object_class = GTK_OBJECT_CLASS (klass);
-	GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
-	GtkBindingSet *binding_set;
+	GObjectClass   *object_class = (GObjectClass *) klass;
+	GtkObjectClass *gtk_object_class = (GtkObjectClass *) klass;
+	GtkWidgetClass *widget_class = (GtkWidgetClass *) klass;
 	
 	basep_widget_parent_class = g_type_class_ref (gtk_window_get_type ());
 
-	klass->move_focus_out = basep_widget_move_focus_out;
 	klass->mode_change = basep_widget_mode_change;
 	klass->state_change = basep_widget_state_change;
 	klass->screen_change = basep_widget_real_screen_change;
@@ -142,7 +140,7 @@ basep_widget_class_init (BasePWidgetClass *klass)
 	widget_class->map = basep_widget_map;
 	widget_class->enter_notify_event = basep_enter_notify;
 	widget_class->leave_notify_event = basep_leave_notify;
-	widget_class->key_press_event = basep_key_press;
+	widget_class->focus_in_event = basep_widget_focus_in_event;
 	widget_class->style_set = basep_style_set;
 
 	gtk_object_class->destroy = basep_widget_destroy;
@@ -153,16 +151,16 @@ basep_widget_class_init (BasePWidgetClass *klass)
         g_object_class_install_property (object_class,
                                         PROP_MODE,
                                         g_param_spec_enum ("mode",
-                                                             _("mode"),
-                                                             _("mode"),
+                                                             _("Mode"),
+                                                             _("Mode of this panel"),
 							    PANEL_TYPE_BASE_PMODE,
                                                             BASEP_EXPLICIT_HIDE,
                                                             G_PARAM_READWRITE));
         g_object_class_install_property (object_class,
                                         PROP_STATE,
                                         g_param_spec_enum ("state",
-                                                             _("state"),
-                                                             _("state"),
+                                                             _("State"),
+                                                             _("Current state of this panel"),
 							     PANEL_TYPE_BASE_PSTATE,
                                                              BASEP_SHOWN,
                                                              G_PARAM_READWRITE));
@@ -170,16 +168,16 @@ basep_widget_class_init (BasePWidgetClass *klass)
         g_object_class_install_property (object_class,
         			  	PROP_HIDEBUTTONS_ENABLED,
                                         g_param_spec_boolean ("hidebuttons_enabled",
-                                                             _("hidebuttons_enabled"),
-                                                             _("hidebuttons_enabled"),
+                                                             _("Hidebuttons enabled"),
+                                                             _("Are hidebuttons (buttons that hide or show the panel) enabled?"),
                                                              TRUE,
                                                              G_PARAM_READWRITE));
 
         g_object_class_install_property (object_class,
         			  	PROP_HIDEBUTTON_PIXMAPS_ENABLED,
                                         g_param_spec_boolean ("hidebutton_pixmaps_enabled",
-                                                             _("hidebutton_pixmaps_enabled"),
-                                                             _("hidebutton_pixmaps_enabled"),
+                                                             _("Hidebutton pixmaps enabled"),
+                                                             _("Hidebuttons have pixmaps"),
                                                              TRUE,
                                                              G_PARAM_READWRITE));
 
@@ -195,17 +193,6 @@ basep_widget_class_init (BasePWidgetClass *klass)
 			      1,
 			      PANEL_TYPE_OBJECT_TYPE); */
 
-	basep_widget_signals[MOVE_FOCUS_OUT_SIGNAL] = 
-		g_signal_new	("move_focus_out",
-				G_TYPE_FROM_CLASS (object_class),
-				G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
-				G_STRUCT_OFFSET (BasePWidgetClass, move_focus_out),
-				NULL,
-				NULL,
-				panel_marshal_VOID__VOID,
-				G_TYPE_NONE,
-				0);
-
 	basep_widget_signals[MODE_CHANGE_SIGNAL] = 
 		g_signal_new	("mode_change",
 			       	G_TYPE_FROM_CLASS (object_class),
@@ -237,21 +224,10 @@ basep_widget_class_init (BasePWidgetClass *klass)
 				G_STRUCT_OFFSET (BasePWidgetClass, screen_change),
 				NULL,
 				NULL,
-				panel_marshal_VOID__INT,
+				g_cclosure_marshal_VOID__INT,
 				G_TYPE_NONE,
 				1,
 				G_TYPE_INT);
-
-	binding_set = gtk_binding_set_by_class (gtk_object_class);
-	gtk_binding_entry_add_signal (binding_set,
-				      GDK_Tab, GDK_CONTROL_MASK,
-				      "move_focus_out", 0);
-	gtk_binding_entry_add_signal (binding_set,
-				      GDK_KP_Tab, GDK_CONTROL_MASK,
-				      "move_focus_out", 0);
-	gtk_binding_entry_add_signal (binding_set,
-				      GDK_ISO_Left_Tab, GDK_CONTROL_MASK,
-				      "move_focus_out", 0);
 }
 
 static void
@@ -558,26 +534,6 @@ basep_widget_size_allocate (GtkWidget *widget,
 	}
 }
 
-static void
-basep_widget_move_focus_out (BasePWidget *basep)
-{
-	if (DRAWER_IS_WIDGET (basep)) {
-		Drawer *drawer = g_object_get_data (G_OBJECT (basep),
-               		                             DRAWER_PANEL_KEY);
-		PanelWidget *panel = PANEL_WIDGET (drawer->button->parent);
-		GtkWidget *parent = panel->panel_parent;
-
-		drawer_widget_close_drawer (DRAWER_WIDGET (basep), parent);
-		drawer->moving_focus = TRUE;
-		gtk_window_present (GTK_WINDOW (parent));
-		gtk_widget_grab_focus (drawer->button);
-	} else {
-		PanelWidget *panel = PANEL_WIDGET (basep->panel);
-
-		panel_widget_focus (panel);
-	}
-}
-
 static void
 basep_widget_mode_change (BasePWidget *basep, BasePMode old_mode)
 {
@@ -588,13 +544,12 @@ basep_widget_mode_change (BasePWidget *basep, BasePMode old_mode)
 static void
 set_tip (GtkWidget *widget, gboolean showing) 
 {
-	if (showing) {
+	if (showing)
 		gtk_tooltips_set_tip (panel_tooltips, widget,
 		      _("Hide this panel"), NULL);
-	} else {
+	else
 		gtk_tooltips_set_tip (panel_tooltips, widget,
 		      _("Show this panel"), NULL);
-	}
 }
 
 static void
@@ -812,8 +767,6 @@ basep_pos_get_hide_pos (BasePWidget *basep,
 static void
 basep_pos_class_init (BasePPosClass *klass)
 {
-	/*GtkObjectClass *object_class = GTK_OBJECT_CLASS(klass);*/
-
 	basep_pos_parent_class = g_type_class_ref (gtk_object_get_type ());
 
 	klass->get_hide_size = basep_pos_get_hide_size;
@@ -827,24 +780,30 @@ basep_pos_instance_init (BasePPos *pos)
 	return;
 }
 
-static gboolean 
-basep_key_press (GtkWidget *widget, GdkEventKey *event)
+static int
+basep_widget_focus_in_event (GtkWidget     *widget,
+			     GdkEventFocus *event)
 {
-	BasePWidget *basep;
-	PanelWidget *panel;
+	BasePWidget *basep = BASEP_WIDGET (widget);
 
-	g_return_val_if_fail (BASEP_IS_WIDGET (widget), FALSE);
+	if (basep->state == BASEP_AUTO_HIDDEN)
+		basep_widget_autoshow (basep);
 
-	basep = BASEP_WIDGET (widget);
+#if 0
+        /* FIXME: When we have focus and hit ctrl-alt-tab with
+         *        Metacity, it does a grab and we a FocusChange
+         *        event because gdk doesn't ignore NotifyGrab
+         *        FocusChange events. See #ifdefed out case
+         *        in gdkevents-x11.c.
+         */
+        else if (basep->state == BASEP_HIDDEN_LEFT ||
+                 basep->state == BASEP_HIDDEN_RIGHT)
+                basep_widget_explicit_show (basep);
+#endif
 
-	g_return_val_if_fail (PANEL_IS_WIDGET (basep->panel), FALSE);
-
-	panel = PANEL_WIDGET (basep->panel);
-	
-	panel_widget_save_key_event (panel, event);
-	return GTK_WIDGET_CLASS (basep_widget_parent_class)->key_press_event (widget, event);
+	return GTK_WIDGET_CLASS (basep_widget_parent_class)->focus_in_event (widget, event);
 }
- 
+
 static gboolean
 basep_leave_notify (GtkWidget *widget,
 		    GdkEventCrossing *event)
@@ -1067,8 +1026,7 @@ basep_widget_do_hiding(BasePWidget *basep, PanelOrient hide_orient,
 			y = move_step(oy,dy,start_time,end_time,cur_time);
 			w = move_step(ow,dw,start_time,end_time,cur_time);
 			h = move_step(oh,dh,start_time,end_time,cur_time);
-			gdk_window_move_resize(wid->window, x,y,w,h);
-			gdk_flush();
+			xstuff_set_pos_size (wid->window, x, y, w, h);
 			g_usleep (1000);
 		}
 
@@ -1186,12 +1144,13 @@ basep_widget_do_showing(BasePWidget *basep, PanelOrient hide_orient,
 			y = move_step(oy,dy,start_time,end_time,cur_time);
 			w = move_step(ow,dw,start_time,end_time,cur_time);
 			h = move_step(oh,dh,start_time,end_time,cur_time);
-			gdk_window_move_resize(wid->window, x,y,w,h);
-			gdk_flush();
+			xstuff_set_pos_size (wid->window, x, y, w, h);
 			
 			/*drawing the entire table flickers, so don't
 			  do it often*/
 			/* FIXME: does it still flicker? */
+			/* FIXME: sometimes this updates, sometimes it doesn't
+			 * see bug #81654 */
 			if (i++ % 10) {
 				/* FIXME: is invalidation needed, and if so it should only be
 				 * partial */
@@ -1509,12 +1468,9 @@ basep_update_frame (BasePWidget *basep)
 }
 
 static void
-basep_back_change (PanelWidget *panel,
-		   PanelBackType type,
-		   char *pixmap,
-		   GdkColor *color,
-		   BasePWidget *basep)
+basep_back_change (PanelWidget *panel, gpointer data)
 {
+	BasePWidget *basep = BASEP_WIDGET (data);
 	basep_update_frame (basep);
 
 	panel_set_frame_colors (panel,
@@ -1526,10 +1482,9 @@ basep_back_change (PanelWidget *panel,
 }
 
 static void
-basep_orient_change (PanelWidget *panel,
-		     GtkOrientation orient,
-		     BasePWidget *basep)
+basep_orient_change (PanelWidget *panel, gpointer data)
 {
+	BasePWidget *basep = BASEP_WIDGET (data);
 	setup_hidebuttons (basep);
 }
 
@@ -2216,88 +2171,69 @@ basep_widget_autohide (gpointer data)
 }
 
 void
-basep_widget_queue_autohide(BasePWidget *basep)
+basep_widget_queue_autohide (BasePWidget *basep)
 {
-        /* check if there's already a timeout set, and delete it if 
-         * there was */
-	if (basep->moving) {
-#ifdef BASEP_WIDGET_DEBUG
-		g_print ("basep_widget_queue_autohide: return 2");
-#endif
+	if (basep->moving)
 		return; 
-	}
 
-        if (basep->enter_notify_timer_tag != 0) {
+        if (basep->enter_notify_timer_tag) {
                 g_source_remove (basep->enter_notify_timer_tag);
                 basep->enter_notify_timer_tag = 0;
 	}
 
-        if (basep->leave_notify_timer_tag != 0) {
+        if (basep->leave_notify_timer_tag) {
                 g_source_remove (basep->leave_notify_timer_tag);
-#ifdef BASEP_WIDGET_DEBUG
-		g_print ("basep_widget_queue_autohide: <timeout removed>\n");
-#endif
+		basep->leave_notify_timer_tag = 0;
 	}
 
-        if ((basep->mode != BASEP_AUTO_HIDE) ||
-            (basep->state != BASEP_SHOWN)) {
-#ifdef BASEP_WIDGET_DEBUG
-		g_print ("basep_widget_queue_autohide: return 1\n");
-#endif
+        if (basep->mode != BASEP_AUTO_HIDE ||
+            basep->state != BASEP_SHOWN)
                 return;
-	}
                 
-	/* set up our delay for popup. */
-	if (global_config.hide_delay == 0) {
+	if (!global_config.hide_delay)
 		basep->leave_notify_timer_tag =
 			g_idle_add (basep_widget_autohide, basep);
-	} else {
+	else
 		basep->leave_notify_timer_tag =
 			g_timeout_add (global_config.hide_delay,
 				       basep_widget_autohide, basep);
-	}
 }
 
 void
 basep_widget_get_menu_pos (BasePWidget *basep,
-			   GtkWidget *menu,
+			   GtkWidget   *menu,
 			   int *x, int *y,
 			   int wx, int wy,
 			   int ww, int wh)
 {
-	GtkRequisition mreq;
-	BasePPosClass *klass = 
-		basep_widget_get_pos_class (basep);
+	BasePPosClass  *klass = basep_widget_get_pos_class (basep);
+	GtkRequisition  mreq;
+	int             monitor_x;
+	int             monitor_y;
+	int             monitor_width;
+	int             monitor_height;
+
 	g_return_if_fail (klass && klass->get_menu_pos);
 
-	gtk_widget_get_child_requisition(menu, &mreq);
+	gtk_widget_get_child_requisition (menu, &mreq);
 
-#ifdef BASEP_WIDGET_DEBUG
-	g_print ("basep_widget_get_menu_pos: menu req: w = %d, h = %d\n", 
-		 mreq.width, mreq.height);
-#endif
+	klass->get_menu_pos (
+			basep, menu, &mreq, x, y, wx, wy, ww, wh);
 
-	klass->get_menu_pos (basep, menu, &mreq,
-			     x, y, wx, wy, 
-			     ww, wh);
+	monitor_x      = multiscreen_x (basep->screen);
+	monitor_y      = multiscreen_y (basep->screen);
+	monitor_width  = multiscreen_width (basep->screen);
+	monitor_height = multiscreen_height (basep->screen);
 
-#ifdef BASEP_WIDGET_DEBUG
-	g_print ("basep_widget_get_menu_pos: x = %d, y = %d\n", *x, *y);
-#endif
+	if (*x + mreq.width > (monitor_x + monitor_width))
+		*x = monitor_x + monitor_width - mreq.width;
+	if (*x < monitor_x)
+		*x = monitor_x;
 
-	if (*x + mreq.width >
-	    multiscreen_x (basep->screen) + multiscreen_width (basep->screen))
-		*x = multiscreen_x (basep->screen) + 
-			multiscreen_width (basep->screen) - mreq.width;
-	if (*x < multiscreen_x (basep->screen))
-		*x = multiscreen_x (basep->screen);
-
-	if (*y + mreq.height >
-	    multiscreen_y (basep->screen) + multiscreen_height (basep->screen))
-		*y = multiscreen_y (basep->screen) + 
-			multiscreen_height (basep->screen) - mreq.height;
-	if (*y < multiscreen_y (basep->screen))
-		*y = multiscreen_y (basep->screen);
+	if (*y + mreq.height > (monitor_y + monitor_height))
+		*y = monitor_y + monitor_height - mreq.height;
+	if (*y < monitor_y)
+		*y = monitor_y;
 }
 
 PanelOrient
@@ -2372,7 +2308,7 @@ basep_widget_set_pos (BasePWidget *basep,
 	g_return_if_fail (klass && klass->set_pos);
 
 	/* first take care of switching screens */
-	newscreen = multiscreen_screen_from_pos (x, y);
+	newscreen = multiscreen_locate_coords (x, y);
 	if (newscreen >= 0 &&
 	    newscreen != basep->screen) {
 		force = TRUE;
@@ -2435,40 +2371,42 @@ static GList *border_list = NULL;
 static ScreenBorders *
 get_borders (int screen)
 {
-	GList *li;
-	ScreenBorders *sb;
+	ScreenBorders *retval;
+	GList         *l;
+
+	for (l = border_list; l; l = l->next) {
+		ScreenBorders *sb = l->data;
 
-	for (li = border_list; li != NULL; li = li->next) {
-		sb = li->data;
 		if (sb->screen == screen)
 			return sb;
 	}
 
-	sb = g_new0 (ScreenBorders, 1);
-	sb->screen = screen;
-	border_list = g_list_prepend (border_list, sb);
+	retval = g_new0 (ScreenBorders, 1);
+	retval->screen  = screen;
+	border_list = g_list_prepend (border_list, retval);
 
-	return sb;
+	return retval;
 }
 
 static void
 basep_calculate_borders (int screen)
 {
-	GSList *li;
 	ScreenBorders *sb;
+	GSList        *l;
 
 	sb = get_borders (screen);
 
-	for (li = panel_list; li != NULL; li = li->next) {
-		PanelData *pd = li->data;
-		BasePWidget *basep;
-		GtkRequisition chreq;
-		BorderEdge edge;
+	for (l = panel_list; l; l = l->next) {
+		PanelData      *pd = l->data;
+		BasePWidget    *basep;
+		GtkRequisition  chreq;
+		BorderEdge      edge;
+		int             size = 0;
 
 		g_assert (pd != NULL);
 
-		if ( ! EDGE_IS_WIDGET (pd->panel) &&
-		     ! ALIGNED_IS_WIDGET (pd->panel))
+		if (!EDGE_IS_WIDGET (pd->panel) &&
+		    !ALIGNED_IS_WIDGET (pd->panel))
 			continue;
 
 		basep = BASEP_WIDGET (pd->panel);
@@ -2481,142 +2419,125 @@ basep_calculate_borders (int screen)
 
 		edge = BORDER_POS (basep->pos)->edge;
 
+		switch (edge) {
+		case BORDER_RIGHT:
+		case BORDER_LEFT:
+			size = chreq.width;
+			break;
+		case BORDER_TOP:
+		case BORDER_BOTTOM:
+			size = chreq.height;
+			break;
+		default:
+			g_assert_not_reached ();
+			break;
+		}
+
 		if (EDGE_IS_WIDGET (basep)) {
 			BasePState state = basep->state;
-			if (PANEL_WIDGET (basep->panel)->orient ==
-			    GTK_ORIENTATION_VERTICAL) {
-				if (sb->borders[edge].left < chreq.width &&
-				    state != BASEP_HIDDEN_RIGHT)
-					sb->borders[edge].left = chreq.width;
-				if (sb->borders[edge].center < chreq.width &&
-				    state != BASEP_HIDDEN_RIGHT &&
-				    state != BASEP_HIDDEN_LEFT)
-					sb->borders[edge].center = chreq.width;
-				if (sb->borders[edge].right < chreq.width &&
-				    state != BASEP_HIDDEN_LEFT)
-					sb->borders[edge].right = chreq.width;
-			} else {
-				if (sb->borders[edge].left < chreq.height &&
-				    state != BASEP_HIDDEN_RIGHT)
-					sb->borders[edge].left = chreq.height;
-				if (sb->borders[edge].center < chreq.height &&
-				    state != BASEP_HIDDEN_RIGHT &&
-				    state != BASEP_HIDDEN_LEFT)
-					sb->borders[edge].center = chreq.height;
-				if (sb->borders[edge].right < chreq.height &&
-				    state != BASEP_HIDDEN_LEFT)
-					sb->borders[edge].right = chreq.height;
+			int        size = 0;
+
+			switch (state) {
+			case BASEP_HIDDEN_RIGHT:
+				sb->borders [edge].right = MAX (sb->borders [edge].right, size);
+				break;
+			case BASEP_HIDDEN_LEFT:
+				sb->borders [edge].left = MAX (sb->borders [edge].left, size);
+				break;
+			default:
+				sb->borders [edge].right  = MAX (sb->borders [edge].right, size);
+				sb->borders [edge].center = MAX (sb->borders [edge].center, size);
+				sb->borders [edge].left   = MAX (sb->borders [edge].left, size);
+				break;
 			}
 		} else /* ALIGNED */ {
-			AlignedAlignment align = ALIGNED_POS(basep->pos)->align;
-			if (PANEL_WIDGET (basep->panel)->orient ==
-			    GTK_ORIENTATION_VERTICAL) {
-				if (align == ALIGNED_LEFT &&
-				    sb->borders[edge].left < chreq.width)
-					sb->borders[edge].left = chreq.width;
-				else if (align == ALIGNED_CENTER &&
-					 sb->borders[edge].center < chreq.width)
-					sb->borders[edge].center = chreq.width;
-				else if (align == ALIGNED_RIGHT &&
-					 sb->borders[edge].right < chreq.width)
-					sb->borders[edge].right = chreq.width;
-			} else {
-				if (align == ALIGNED_LEFT &&
-				    sb->borders[edge].left < chreq.height)
-					sb->borders[edge].left = chreq.height;
-				else if (align == ALIGNED_CENTER &&
-					 sb->borders[edge].center < chreq.height)
-					sb->borders[edge].center = chreq.height;
-				else if (align == ALIGNED_RIGHT &&
-					 sb->borders[edge].right < chreq.height)
-					sb->borders[edge].right = chreq.height;
+			AlignedAlignment align = ALIGNED_POS (basep->pos)->align;
+
+			switch (align) {
+			case ALIGNED_LEFT:
+				sb->borders [edge].left = MAX (sb->borders [edge].left, size);
+				break;
+			case ALIGNED_RIGHT:
+				sb->borders [edge].right = MAX (sb->borders [edge].right, size);
+				break;
+			case ALIGNED_CENTER:
+				sb->borders [edge].center = MAX (sb->borders [edge].center, size);
+				break;
+			default:
+				g_assert_not_reached ();
+				break;
 			}
 		}
 	}
 }
 
 static int
-border_max (ScreenBorders *sb, BorderEdge edge)
+border_max (ScreenBorders *sb,
+	    BorderEdge     edge)
 {
-	return MAX (sb->borders[edge].center,
-		    MAX (sb->borders[edge].left, sb->borders[edge].right));
+	return MAX (sb->borders [edge].center,
+			MAX (sb->borders [edge].left,
+			     sb->borders [edge].right));
 }
 
 void
 basep_border_recalc (int screen)
 {
-	int i;
-	GSList *li;
 	ScreenBorders *sb;
-	ScreenBorders old;
+	ScreenBorders  old;
+	GSList        *l;
+	int            i;
 
 	sb = get_borders (screen);
 
 	memcpy (&old, sb, sizeof (ScreenBorders));
 
 	for (i = 0; i < 4; i++) {
-		sb->borders[i].left = 0;
-		sb->borders[i].center = 0;
-		sb->borders[i].right = 0;
+		sb->borders [i].left = 0;
+		sb->borders [i].center = 0;
+		sb->borders [i].right = 0;
 	}
 
 	basep_calculate_borders (screen);
 
-	sb->left = border_max (sb, BORDER_LEFT);
-	sb->right = border_max (sb, BORDER_RIGHT);
-	sb->top = border_max (sb, BORDER_TOP) +
-		foobar_widget_get_height (screen);
+	sb->left   = border_max (sb, BORDER_LEFT);
+	sb->right  = border_max (sb, BORDER_RIGHT);
 	sb->bottom = border_max (sb, BORDER_BOTTOM);
+	sb->top    = border_max (sb, BORDER_TOP) +
+				foobar_widget_get_height (screen);
 
-#ifdef BASEP_WIDGET_DEBUG
-	g_print ("[basep_border_recalc] SCREEN %d (%d %d %d %d)\n", screen,
-		 sb->left, sb->right, sb->top, sb->bottom);
-#endif
+	if (!memcmp (&old, sb, sizeof (ScreenBorders)))
+		return;
 
-	if (memcmp (&old, sb, sizeof (ScreenBorders)) != 0) {
-		for (li = panel_list; li != NULL; li = li->next) {
-			PanelData *pd = li->data;
-			GtkWidget *panel;
+	for (l = panel_list; l; l = l->next) {
+		PanelData *pdata = l->data;
+		GtkWidget *panel;
 
-			g_assert (pd != NULL);
+		g_assert (pdata != NULL);
 
-			panel = pd->panel;
+		panel = pdata->panel;
 
-			if (BORDER_IS_WIDGET (panel) &&
-			    BASEP_WIDGET (panel)->screen == screen) {
-				gtk_widget_queue_resize (panel);
-			}
-		}
-
-		if (sb->left != old.left ||
-		    sb->right != old.right ||
-		    sb->top != old.top ||
-		    sb->bottom != old.bottom)
-			xstuff_setup_desktop_area (screen,
-						   sb->left,
-						   sb->right,
-						   sb->top,
-						   sb->bottom);
+		if (BORDER_IS_WIDGET (panel) &&
+		    BASEP_WIDGET (panel)->screen == screen)
+			gtk_widget_queue_resize (panel);
 	}
 }
 
-static guint queue_recalc_id = 0;
-static GList *recalc_list = NULL;
+static guint  queue_recalc_id = 0;
+static GList *pending_recalc_list = NULL;
 
 static gboolean
 queue_recalc_handler (gpointer data)
 {
-	GList *list, *li;
+	GList *list, *l;
 
 	queue_recalc_id = 0;
 
-	list = recalc_list;
-	recalc_list = NULL;
+	list = pending_recalc_list;
+	pending_recalc_list = NULL;
 
-	for (li = list; li != NULL; li = li->next) {
-		int screen = GPOINTER_TO_INT (li->data);
-		basep_border_recalc (screen);
-	}
+	for (l = list; l; l = l->next)
+		basep_border_recalc (GPOINTER_TO_INT (l->data));
 
 	g_list_free (list);
 
@@ -2626,9 +2547,10 @@ queue_recalc_handler (gpointer data)
 void
 basep_border_queue_recalc (int screen)
 {
-	if (!g_list_find (recalc_list, GINT_TO_POINTER (screen)))
-		recalc_list = g_list_prepend (recalc_list,
-					      GINT_TO_POINTER (screen));
+	if (!g_list_find (pending_recalc_list, GINT_TO_POINTER (screen)))
+		pending_recalc_list =
+			g_list_prepend (pending_recalc_list,
+					GINT_TO_POINTER (screen));
 
 	if (!queue_recalc_id)
 		queue_recalc_id = g_idle_add (queue_recalc_handler, NULL);
