@@ -88,6 +88,7 @@ void on_preview_configure_event (GtkWidget *drawing_area,
 				 gpointer data);
 void on_ok_button_clicked (GtkWidget *widget, gpointer data);
 void on_cancel_button_clicked (GtkWidget *widget, gpointer data);
+void on_help_button_clicked (GtkWidget *widget, gpointer data);
 int on_toplevel_key_press_event (GtkWidget *widget, GdkEventKey *key);
 
 /* some local prototypes */
@@ -192,6 +193,7 @@ nibble_on_file (const char *file)
 {
 	GtkWidget *dialog;
 	FILE *fp;
+	mode_t old_mask;
 
 	if (access (file, F_OK) == 0) {
 		dialog = gtk_message_dialog_new
@@ -201,9 +203,14 @@ nibble_on_file (const char *file)
 			 GTK_BUTTONS_YES_NO,
 			 _("File %s already exists. Overwrite?"),
 			 file);
-		if (gtk_dialog_run (GTK_DIALOG (dialog)) != GTK_RESPONSE_YES)
+		if (gtk_dialog_run (GTK_DIALOG (dialog)) != GTK_RESPONSE_YES) {
+			gtk_widget_destroy (dialog);
 			return NULL;
+		}
 	}
+
+	old_mask = umask(077);
+
 	fp = fopen (file, "w");
 	if (fp == NULL) {
 		dialog = gtk_message_dialog_new
@@ -216,8 +223,12 @@ nibble_on_file (const char *file)
 			   "Please check your permissions of "
 			   "the parent directory"), file);
 		gtk_dialog_run (GTK_DIALOG (dialog));
+		gtk_widget_destroy (dialog);
+
+		umask(old_mask);
 		return NULL;
 	}
+	umask(old_mask);
 	return fp;
 }
 
@@ -240,6 +251,7 @@ save_to_file (FILE *fp, const gchar *file, gboolean gui_errors)
 				 GTK_BUTTONS_OK,
 				 "%s", error);
 			gtk_dialog_run (GTK_DIALOG (dialog));
+			gtk_widget_destroy (dialog);
 		}
 		fclose (fp);
 		unlink (file);
@@ -255,6 +267,7 @@ start_temporary (void)
 {
 	char *dir;
 	char *file = NULL;
+	mode_t old_mask;
 
 	if (temporary_file != NULL) {
 		if (access (temporary_file, F_OK) == 0)
@@ -284,12 +297,14 @@ start_temporary (void)
 			g_free (dir);
 		dir = g_strdup_printf ("/tmp/gnome-panel-screenshot-%d",
 				       rand ());
-	} while (mkdir (dir, 0755) < 0);
+	} while (mkdir (dir, 0700) < 0);
 
 	file = add_file_to_path (dir);
 
 	g_free (dir);
 
+	old_mask = umask(077);
+
 	temporary_pid = fork ();
 
 	if (temporary_pid == 0) {
@@ -309,11 +324,14 @@ start_temporary (void)
 		    ! save_to_file (fp, file, TRUE)) {
 			g_free (file);
 			temporary_pid = 0;
+
+			umask(old_mask);
 			return;
 		}
 		temporary_pid = 0;
 	}
 
+	umask(old_mask);
 	temporary_file = file;
 }
 
@@ -543,18 +561,23 @@ print_pixbuf (void)
 static gchar *
 add_file_to_path (const gchar *path)
 {
-	gchar *retval;
-	gint i = 1;
+	char *retval;
+	char *tmp;
+	int   i = 1;
 
 	if (class_name) {
 		/* translators: this is the file that gets made up with the screenshot if a specific window is taken */
-		retval = g_strdup_printf (_("%s%cScreenshot-%s.png"), path,
-					  G_DIR_SEPARATOR, class_name);
+		tmp = g_strdup_printf (_("%s%cScreenshot-%s.png"), path,
+				       G_DIR_SEPARATOR, class_name);
+		retval = g_filename_from_utf8 (tmp, -1, NULL, NULL, NULL);
+		g_free (tmp);
 	}
 	else {
 		/* translators: this is the file that gets made up with the screenshot if the entire screen is taken */
-		retval = g_strdup_printf (_("%s%cScreenshot.png"), path,
-					  G_DIR_SEPARATOR);
+		tmp = g_strdup_printf (_("%s%cScreenshot.png"), path,
+				       G_DIR_SEPARATOR);
+		retval = g_filename_from_utf8 (tmp, -1, NULL, NULL, NULL);
+		g_free (tmp);
 	}
 	
 	do {
@@ -568,13 +591,17 @@ add_file_to_path (const gchar *path)
 
 		if (class_name) {
 			/* translators: this is the file that gets made up with the screenshot if a specific window is taken */
-			retval = g_strdup_printf (_("%s%cScreenshot-%s-%d.png"), path,
-						  G_DIR_SEPARATOR, class_name, i);
+			tmp = g_strdup_printf (_("%s%cScreenshot-%s-%d.png"), path,
+					       G_DIR_SEPARATOR, class_name, i);
+			retval = g_filename_from_utf8 (tmp, -1, NULL, NULL, NULL);
+			g_free (tmp);
 		}
 		else {
 			/* translators: this is the file that gets made up with the screenshot if the entire screen is taken */
-			retval = g_strdup_printf (_("%s%cScreenshot-%d.png"), path,
-						  G_DIR_SEPARATOR, i);
+			tmp = g_strdup_printf (_("%s%cScreenshot-%d.png"), path,
+					       G_DIR_SEPARATOR, i);
+			retval = g_filename_from_utf8 (tmp, -1, NULL, NULL, NULL);
+			g_free (tmp);
 		}
 		
 		i++;
@@ -681,8 +708,6 @@ gimme_file (const char *filename)
 		fclose (fp);
 
 		if (rename (temporary_file, filename) == 0) {
-			g_free (temporary_file);
-			temporary_file = NULL;
 			return TRUE;
 		}
 		infd = open (temporary_file, O_RDONLY);
@@ -691,7 +716,7 @@ gimme_file (const char *filename)
 			return FALSE;
 		}
 
-		outfd = open (filename, O_CREAT|O_TRUNC|O_WRONLY, 0644);
+		outfd = open (filename, O_CREAT|O_TRUNC|O_WRONLY, 0600);
 		if (outfd < 0) {
 			GtkWidget *dialog;
 			dialog = gtk_message_dialog_new
@@ -704,6 +729,7 @@ gimme_file (const char *filename)
 				   "Please check your permissions of "
 				   "the parent directory"), filename);
 			gtk_dialog_run (GTK_DIALOG (dialog));
+			gtk_widget_destroy (dialog);
 			close (infd);
 			return FALSE;
 		}
@@ -722,6 +748,7 @@ gimme_file (const char *filename)
 					 _("Not enough room to write file %s"),
 					 filename);
 				gtk_dialog_run (GTK_DIALOG (dialog));
+				gtk_widget_destroy (dialog);
 				return FALSE;
 			}
 		}
@@ -795,6 +822,37 @@ on_cancel_button_clicked (GtkWidget *widget, gpointer data)
 	gtk_main_quit ();
 }
 
+void
+on_help_button_clicked (GtkWidget *widget, gpointer data) 
+{
+	
+	GError *error = NULL;
+
+        gnome_help_display_desktop (NULL, "user-guide", 
+				    "wgoeditmainmenu.xml", "goseditmainmenu-51", 
+				    &error);
+	
+	if (error) {
+		GtkWidget *dialog;
+
+                dialog = gtk_message_dialog_new (GTK_WINDOW (toplevel),
+                        GTK_DIALOG_DESTROY_WITH_PARENT,
+                        GTK_MESSAGE_ERROR,
+                        GTK_BUTTONS_CLOSE,
+                        _("There was an error displaying help: \n%s"),
+                        error->message);
+
+                g_signal_connect (G_OBJECT (dialog),
+                        "response",
+                        G_CALLBACK (gtk_widget_destroy), NULL);
+                gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
+                gtk_widget_show (dialog);
+                g_error_free (error);
+
+        }
+
+}
+
 int
 on_toplevel_key_press_event (GtkWidget *widget,
 			     GdkEventKey *key)
@@ -1012,7 +1070,7 @@ main (int argc, char *argv[])
 	GConfClient *gconf_client;
 	GnomeClient *client;
 	struct stat s;
-	gchar *file;
+	gchar *file, *window_icon;
 	gboolean window = FALSE;
 	gint width, height; 
 	guint delay = 0;
@@ -1047,11 +1105,11 @@ main (int argc, char *argv[])
 	else
 		take_screen_shot ();
 
-	if (g_file_test ("gnome-panel-screenshot.glade2", G_FILE_TEST_EXISTS)) {
-		xml = glade_xml_new ("gnome-panel-screenshot.glade2", NULL, NULL);
+	if (g_file_test ("gnome-panel-screenshot.glade", G_FILE_TEST_EXISTS)) {
+		xml = glade_xml_new ("gnome-panel-screenshot.glade", NULL, NULL);
 	}
 	if (xml == NULL) {
-		xml = glade_xml_new (GLADEDIR "/gnome-panel-screenshot.glade2",
+		xml = glade_xml_new (GLADEDIR "/gnome-panel-screenshot.glade",
 				     NULL, NULL);
 	}
 	if (xml == NULL) {
@@ -1064,6 +1122,7 @@ main (int argc, char *argv[])
 			 _("Glade file for the screenshot program is missing.\n"
 			   "Please check your installation of gnome-panel"));
 		gtk_dialog_run (GTK_DIALOG (dialog));
+		gtk_widget_destroy (dialog);
 		exit (1);
 	}
 	glade_xml_signal_autoconnect (xml);
@@ -1086,6 +1145,7 @@ main (int argc, char *argv[])
 			_("Unable to take a screenshot of "
 			   "the current desktop."));
 		gtk_dialog_run (GTK_DIALOG (dialog));
+		gtk_widget_destroy (dialog);
 		exit (1);
 	}
 
@@ -1100,6 +1160,14 @@ main (int argc, char *argv[])
 	preview = glade_xml_get_widget (xml, "preview");
 	save_entry = glade_xml_get_widget (xml, "save_entry");
 
+	window_icon = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_PIXMAP, 
+						 "gnome-screenshot.png", TRUE, NULL);
+	if (window_icon) {
+		gnome_window_icon_set_from_file (GTK_WINDOW (toplevel), window_icon);
+		g_free (window_icon);
+	}
+
+	
 	gtk_window_set_default_size (GTK_WINDOW (toplevel), width * 2, -1);
 	gtk_widget_set_size_request (preview, width, height);
 	gtk_aspect_frame_set (GTK_ASPECT_FRAME (frame), 0.5, 0.5,
