@@ -33,6 +33,7 @@
 #include "panel-util.h"
 #include "panel-config-global.h"
 #include "panel-gconf.h"
+#include "panel-main.h"
 #include "session.h"
 
 #include "quick-desktop-reader.h"
@@ -42,12 +43,12 @@
 static void properties_apply (Launcher *launcher);
 static void launcher_save    (Launcher *launcher);
 
+static GSList *launchers_to_hoard = NULL;
+
 extern GtkTooltips *panel_tooltips;
 
 extern GSList *applets;
 
-static char *default_app_pixmap = NULL;
-
 extern GlobalConfig global_config;
 extern gboolean commie_mode;
 
@@ -87,55 +88,49 @@ launch_url (Launcher *launcher)
 
 
 static void
-launch (Launcher *launcher)
+launch_cb (GtkWidget *widget,
+	   Launcher  *launcher)
 {
 	GnomeDesktopItem *item;
-	GnomeDesktopItemType type;
 
-	g_return_if_fail(launcher != NULL);
-	g_return_if_fail(launcher->ditem != NULL);
+	g_return_if_fail (launcher != NULL);
+	g_return_if_fail (launcher->ditem != NULL);
 
 	item = launcher->ditem;
-	type = gnome_desktop_item_get_entry_type (item);
 
-	if (type == GNOME_DESKTOP_ITEM_TYPE_LINK) {
+	if (gnome_desktop_item_get_entry_type (item) == GNOME_DESKTOP_ITEM_TYPE_LINK)
 		launch_url (launcher);
-	} else {
+
+	else {
 		GError *error = NULL;
-		gnome_desktop_item_launch (item,
-					   NULL /* file_list */,
-					   0 /* flags */,
-					   &error);
-		if (error != NULL) {
+
+		gnome_desktop_item_launch (item, NULL, 0, &error);
+		if (error) {
 			panel_error_dialog ("cannot_launch_icon",
 					    _("<b>Cannot launch icon</b>\n\n"
-					      "Details: %s"),
-					    error->message);
+					      "Details: %s"), error->message);
 			g_clear_error (&error);
 		}
 	}
 	
-	if(global_config.drawer_auto_close) {
-		GtkWidget *parent =
-			PANEL_WIDGET(launcher->button->parent)->panel_parent;
-		g_return_if_fail(parent!=NULL);
-		if(DRAWER_IS_WIDGET(parent)) {
-			BasePWidget *basep = BASEP_WIDGET(parent);
-			GtkWidget *grandparent = PANEL_WIDGET(basep->panel)->master_widget->parent;
-			GtkWidget *grandparentw =
-				PANEL_WIDGET(grandparent)->panel_parent;
-			drawer_widget_close_drawer (DRAWER_WIDGET (parent),
-						    grandparentw);
+	if (global_config.drawer_auto_close) {
+		GtkWidget *parent;
+
+		parent = PANEL_WIDGET (launcher->button->parent)->panel_parent;
+		if (DRAWER_IS_WIDGET (parent)) {
+			GtkWidget *grandparent;
+
+			grandparent = PANEL_WIDGET (
+						PANEL_WIDGET (
+							BASEP_WIDGET (parent)->panel
+							     )->master_widget->parent
+						     )->panel_parent;
+			drawer_widget_close_drawer (
+					DRAWER_WIDGET (parent), grandparent);
 		}
 	}
 }
 
-static void
-launch_cb (GtkWidget *widget, gpointer data)
-{
-	launch (data);
-}
-
 static void
 drag_data_received_cb (GtkWidget        *widget,
 		       GdkDragContext   *context,
@@ -179,6 +174,8 @@ free_launcher (gpointer data)
 {
 	Launcher *launcher = data;
 
+	launchers_to_hoard = g_slist_remove (launchers_to_hoard, launcher);
+
 	gnome_desktop_item_unref (launcher->ditem);
 	launcher->ditem = NULL;
 
@@ -317,39 +314,44 @@ static Launcher *
 create_launcher (const char *parameters, GnomeDesktopItem *ditem)
 {
 	Launcher *launcher;
+	GError   *error = NULL;
+
         static GtkTargetEntry dnd_targets[] = {
 		{ "application/x-panel-icon-internal", 0, TARGET_ICON_INTERNAL },
 		{ "text/uri-list", 0, TARGET_URI_LIST }
 	};
 
-	if (!default_app_pixmap)
-		default_app_pixmap = panel_pixmap_discovery ("gnome-unknown.png",
-							     FALSE /* fallback */);
-	if (ditem == NULL) {
-		if (parameters == NULL) {
+	if (!ditem) {
+		if (!parameters) {
+			g_printerr (_("No URI provided for panel launcher desktop file\n"));
 			return NULL;
 		}
 
-		ditem = gnome_desktop_item_new_from_uri (parameters,
-							 0 /* flags */,
-							 NULL /* error */);
+		ditem = gnome_desktop_item_new_from_uri (parameters, 0, &error);
+	}
 
-		if (ditem == NULL) {
-			gchar *entry;
+	if (!ditem) {
+		char *entry;
 
-			entry = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_DATADIR, 
-							   parameters, TRUE, NULL);
-
-			if (!entry)
-				return NULL;
+		entry = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_DATADIR, 
+						   parameters, TRUE, NULL);
 
+		if (entry != NULL) {
 			ditem = gnome_desktop_item_new_from_file (entry, 0, NULL);
-
 			g_free (entry);
 		}
 	}
-	if (ditem == NULL)
+
+	if (!ditem) {
+		g_printerr (_("Unable to open desktop file %s for panel launcher%s%s\n"),
+			    parameters,
+			    error ? ": " : "",
+			    error ? error->message : "");
+		if (error)
+			g_error_free (error);
+
 		return NULL; /*button is null*/
+	}
 
 	launcher = g_new0 (Launcher, 1);
 
@@ -386,30 +388,20 @@ create_launcher (const char *parameters, GnomeDesktopItem *ditem)
 	gtk_drag_dest_set (GTK_WIDGET (launcher->button),
 			   0, NULL, 0, 0);
 
-	g_signal_connect (G_OBJECT(launcher->button), "drag_data_get",
-			   G_CALLBACK(drag_data_get_cb),
-			   launcher);
-	g_signal_connect (G_OBJECT(launcher->button), "drag_data_received",
-			   G_CALLBACK(drag_data_received_cb),
-			   launcher);
-	g_signal_connect (G_OBJECT(launcher->button), "drag_motion",
-			   G_CALLBACK(drag_motion_cb),
-			   launcher);
-	g_signal_connect (G_OBJECT(launcher->button), "drag_drop",
-			   G_CALLBACK(drag_drop_cb),
-			   launcher);
-	g_signal_connect (G_OBJECT(launcher->button), "drag_leave",
-			   G_CALLBACK(drag_leave_cb),
-			   launcher);
-
-
-	g_signal_connect (G_OBJECT(launcher->button), "clicked",
-			    (GtkSignalFunc) launch_cb,
-			    launcher);
-	
-	g_signal_connect (G_OBJECT(launcher->button), "destroy",
-			    G_CALLBACK(destroy_launcher),
-			    launcher);
+	g_signal_connect (launcher->button, "drag_data_get",
+			   G_CALLBACK (drag_data_get_cb), launcher);
+	g_signal_connect (launcher->button, "drag_data_received",
+			   G_CALLBACK (drag_data_received_cb), launcher);
+	g_signal_connect (launcher->button, "drag_motion",
+			   G_CALLBACK (drag_motion_cb), launcher);
+	g_signal_connect (launcher->button, "drag_drop",
+			   G_CALLBACK (drag_drop_cb), launcher);
+	g_signal_connect (launcher->button, "drag_leave",
+			   G_CALLBACK (drag_leave_cb), launcher);
+	g_signal_connect (launcher->button, "clicked",
+			  G_CALLBACK (launch_cb), launcher);
+	g_signal_connect (launcher->button, "destroy",
+			  G_CALLBACK (destroy_launcher), launcher);
 
 	launcher->ditem = ditem;
 
@@ -423,7 +415,7 @@ setup_button (Launcher *launcher)
 	const char *name;
 	const char *docpath;
 	char *str;
-	char *icon;
+	const char *icon;
 	
 	g_return_if_fail (launcher != NULL);
 
@@ -433,33 +425,33 @@ setup_button (Launcher *launcher)
 						       GNOME_DESKTOP_ITEM_COMMENT);
 
 	/* Setup tooltip */
-	if (comment != NULL)
+	if ( ! string_empty (comment))
 		str = g_strdup_printf ("%s\n%s", name, comment);
 	else
 		str = g_strdup (name);
 	gtk_tooltips_set_tip (panel_tooltips, launcher->button,
 			      str, NULL);
+
+	/* Setup accessible name */
+	add_atk_name_desc (launcher->button, str, NULL);
+
 	g_free (str);
 
 	/* Setup text */
 	button_widget_set_text (BUTTON_WIDGET (launcher->button), name);
 
 	/* Setup icon */
-	icon = gnome_desktop_item_get_icon (launcher->ditem);
-	if (icon == NULL ||
-	    ! button_widget_set_pixmap (BUTTON_WIDGET (launcher->button),
-					icon, -1))
-		button_widget_set_pixmap (BUTTON_WIDGET (launcher->button),
-					  default_app_pixmap, -1);
-	g_free (icon);
+	icon = gnome_desktop_item_get_string (launcher->ditem,
+					      GNOME_DESKTOP_ITEM_ICON);
+	button_widget_set_pixmap (BUTTON_WIDGET (launcher->button), icon, -1);
 
 	/* Setup help */
 	docpath = gnome_desktop_item_get_string (launcher->ditem,
-						 "DocPath");
+						 "X-GNOME-DocPath");
 
 	panel_applet_remove_callback (launcher->info, "help_on_app");
 
-	if (docpath != NULL) {
+	if (docpath && *docpath != '\0') {
 		char *title;
 
 		title = g_strdup_printf (_("Help on %s Application"), name);
@@ -542,7 +534,7 @@ window_response (GtkWidget *w, int response, gpointer data)
 	Launcher *launcher = data;
 
 	if (response == GTK_RESPONSE_HELP) {
-		panel_show_help ("launchers", NULL);
+		panel_show_help ("wgospanel.xml", "gospanel-52");
 	} else if (response == REVERT_BUTTON) { /* revert */
 		if (launcher->ditem != NULL)
 			gnome_desktop_item_unref (launcher->ditem);
@@ -581,7 +573,7 @@ create_properties_dialog (Launcher *launcher)
 
 	dialog = gtk_dialog_new ();
 
-	gtk_window_set_title (GTK_WINDOW (dialog), _("Launcher properties"));
+	gtk_window_set_title (GTK_WINDOW (dialog), _("Launcher Properties"));
 
 	help = gtk_dialog_add_button (
 			GTK_DIALOG (dialog), GTK_STOCK_HELP, GTK_RESPONSE_HELP);
@@ -686,7 +678,11 @@ launcher_load_from_gconf (PanelWidget *panel_widget,
 	temp_key = panel_gconf_full_key (
 			PANEL_GCONF_OBJECTS, profile, gconf_key, "launcher_location");
 	launcher_location = gconf_client_get_string (client, temp_key, NULL);
-
+	if (!launcher_location) {
+		g_printerr (_("Key %s is not set, can't load launcher\n"), temp_key);
+		return;
+	}
+        
 	load_launcher_applet (launcher_location, panel_widget, position, TRUE, gconf_key);
 
 	g_free (launcher_location);
@@ -724,7 +720,7 @@ load_launcher_applet_full (const char       *params,
 		panel_applet_add_callback (launcher->info,
 					   "properties",
 					   GTK_STOCK_PROPERTIES,
-					   _("Properties..."));
+					   _("Properties"));
 
 	panel_applet_add_callback (launcher->info, "help", GTK_STOCK_HELP, _("Help"));
 
@@ -755,7 +751,7 @@ really_add_launcher (GtkWidget *dialog, int response, gpointer data)
 
 		panel_config_sync_schedule ();
 	} else if (response == GTK_RESPONSE_HELP) {
-		panel_show_help ("launchers", "LAUNCHERS");
+		panel_show_help ("wgospanel.xml", "gospanel-52");
 		/* just return as we don't want to close */
 		return;
 	}
@@ -764,22 +760,23 @@ really_add_launcher (GtkWidget *dialog, int response, gpointer data)
 }
 
 void
-ask_about_launcher (const char *file, PanelWidget *panel, int pos, gboolean exactpos)
+ask_about_launcher (const char  *file,
+		    PanelWidget *panel,
+		    int          pos,
+		    gboolean     exactpos)
 {
 	GtkWidget *dialog;
 	GnomeDItemEdit *dee;
 	GnomeDesktopItem *ditem;
 
-	dialog = gtk_dialog_new_with_buttons (_("Create launcher applet"),
-					      NULL /* parent */,
-					      0 /* flags */,
-					      GTK_STOCK_HELP,
-					      GTK_RESPONSE_HELP,
-					      GTK_STOCK_CANCEL,
-					      GTK_RESPONSE_CANCEL,
-					      GTK_STOCK_OK,
-					      GTK_RESPONSE_OK,
-					      NULL);
+	dialog = gtk_dialog_new_with_buttons (
+				_("Create Launcher"),
+				GTK_WINDOW (panel->panel_parent),
+				0 /* flags */,
+				GTK_STOCK_HELP, GTK_RESPONSE_HELP,
+				GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+				GTK_STOCK_OK, GTK_RESPONSE_OK,
+				NULL);
 
 	gtk_window_set_wmclass (GTK_WINDOW (dialog),
 				"create_launcher", "Panel");
@@ -818,7 +815,8 @@ ditem_set_icon (GnomeDesktopItem *ditem, const char *icon)
 {
 	if (icon != NULL &&
 	    icon[0] != G_DIR_SEPARATOR) {
-		char *full = gnome_desktop_item_find_icon (icon,
+		char *full = gnome_desktop_item_find_icon (panel_icon_loader,
+							   icon,
 							   48 /* desired size */,
 							   0 /* flags */);
 		if (full != NULL) {
@@ -998,6 +996,25 @@ launcher_save (Launcher *launcher)
 	}
 }
 
+static gboolean
+launcher_idle_hoard (void)
+{
+	Launcher *launcher;
+
+	if (!launchers_to_hoard)
+		return FALSE;
+
+	launcher = launchers_to_hoard->data;
+	launchers_to_hoard = g_slist_delete_link (
+					launchers_to_hoard, launchers_to_hoard);
+
+	launcher_save (launcher);
+	launcher_save_to_gconf (launcher, launcher->info->gconf_key);
+
+	return launchers_to_hoard ? TRUE : FALSE;
+}
+
+
 void
 launcher_hoard (Launcher *launcher)
 {
@@ -1006,9 +1023,10 @@ launcher_hoard (Launcher *launcher)
 
 	gnome_desktop_item_set_location (launcher->ditem, NULL);
 
-	launcher_save (launcher);
+	if (!launchers_to_hoard)
+		g_idle_add ((GSourceFunc) launcher_idle_hoard, NULL);
 
-	launcher_save_to_gconf (launcher, launcher->info->gconf_key);
+	launchers_to_hoard = g_slist_prepend (launchers_to_hoard, launcher);
 }
 
 Launcher *
@@ -1040,3 +1058,24 @@ find_launcher (const char *path)
 
 	return NULL;
 }
+
+void
+launcher_show_help (Launcher *launcher)
+{
+	GError     *error = NULL;
+	const char *docpath;
+
+	if (!launcher->ditem)
+		return;
+
+	docpath = gnome_desktop_item_get_string (
+				launcher->ditem, "X-GNOME-DocPath");
+	panel_show_gnome_kde_help (docpath, &error);
+	if (error) {
+		panel_error_dialog (
+			"cannot_show_gnome_kde_help",
+			_("<b>Cannot display help document</b>\n\nDetails: %s"),
+			error->message);
+		g_error_free (error);
+	}
+}
