@@ -44,10 +44,10 @@ panel_show_help (const char *doc_name, const char *linkid)
 {
 	GError *error = NULL;
 
-	if (!gnome_help_display_desktop (NULL, "panel", doc_name, linkid, &error)) {
+	if (!gnome_help_display_desktop (NULL, "user-guide", doc_name, linkid, &error)) {
 		panel_error_dialog ("cannot_show_help",
 				    _("<b>Cannot display help document</b>\n\n"
-				      "Details: %s"), error->message);
+				      "Details: %s"), error != NULL ? error->message : "");
 		g_clear_error (&error);
 	}
 }
@@ -205,6 +205,7 @@ create_text_entry(GtkWidget *table,
 			 GTK_EXPAND | GTK_FILL | GTK_SHRINK,
 			 GTK_FILL | GTK_SHRINK,
 			 GNOME_PAD_SMALL, GNOME_PAD_SMALL);
+	set_relation (entry, GTK_LABEL (wlabel), 1);
 
 	if(func) {
 		g_object_set_data (G_OBJECT (t), "update_function", func);
@@ -243,6 +244,7 @@ create_icon_entry(GtkWidget *table,
 	if (text != NULL)
 		gnome_icon_entry_set_filename (GNOME_ICON_ENTRY(entry), text);
 
+	set_relation (entry, GTK_LABEL (wlabel), 1);
 	gtk_table_attach(GTK_TABLE(table), entry,
 			 cols, cole, 1, 2,
 			 GTK_EXPAND | GTK_FILL | GTK_SHRINK,
@@ -574,14 +576,15 @@ panel_parse_accelerator (GlobalConfigKey *key)
 	key->keysym = 0;
 	key->state = 0;
 
-	if (!key->str || !key->str [0] ||
-	    !strcmp (key->str, "Disabled") ||
-	    !strcmp (key->str, _("Disabled")))
+	if (key->str == NULL ||
+	    key->str[0] == '\0' ||
+	    strcmp (key->str, "Disabled") == 0 ||
+	    strcmp (key->str, _("Disabled")) == 0)
 		return FALSE;
 
 	gtk_accelerator_parse (key->str, &key->keysym, &key->state);
 
-	if (!key->keysym)
+	if (key->keysym == 0)
 		return FALSE;
 
 	return TRUE;
@@ -597,6 +600,35 @@ convert_keysym_state_to_string(guint keysym,
 	return gtk_accelerator_name (keysym, state);
 }
 
+static GtkWidget *
+panel_dialog (GtkWidget *parent,
+	      int type,
+	      const char *class,
+	      const char *str)
+{
+	GtkWidget *w;
+
+	w = gtk_message_dialog_new ((GtkWindow *) parent, 0, type,
+				    GTK_BUTTONS_OK, "foo");
+	gtk_widget_add_events (w, GDK_KEY_PRESS_MASK);
+	g_signal_connect (G_OBJECT (w), "event",
+			  G_CALLBACK (panel_dialog_window_event), NULL);
+	gtk_window_set_wmclass (GTK_WINDOW (w),
+				class, "Panel");
+
+	gtk_label_set_markup (GTK_LABEL (GTK_MESSAGE_DIALOG (w)->label), str);
+
+	gtk_widget_show_all (w);
+
+	/* FIXME: this is ugly and makes it bad to run gtk_dialog_run
+	 * after this function */
+	g_signal_connect_swapped (G_OBJECT (w), "response",
+				  G_CALLBACK (gtk_widget_destroy),
+				  G_OBJECT (w));
+
+	return w;
+}
+
 GtkWidget *
 panel_error_dialog (const char *class,
 		    const char *format,
@@ -615,24 +647,8 @@ panel_error_dialog (const char *class,
 		va_end (ap);
 	}
 
-	w = gtk_message_dialog_new (NULL, 0, GTK_MESSAGE_ERROR,
-				    GTK_BUTTONS_OK, "foo");
-	gtk_widget_add_events (w, GDK_KEY_PRESS_MASK);
-	g_signal_connect (G_OBJECT (w), "event",
-			  G_CALLBACK (panel_dialog_window_event), NULL);
-	gtk_window_set_wmclass (GTK_WINDOW (w),
-				class, "Panel");
-
-	gtk_label_set_markup (GTK_LABEL (GTK_MESSAGE_DIALOG (w)->label), s);
-
+	w = panel_dialog (NULL, GTK_MESSAGE_ERROR, class, s);
 	g_free (s);
-
-	gtk_widget_show_all (w);
-
-	g_signal_connect_swapped (G_OBJECT (w), "response",
-				   G_CALLBACK (gtk_widget_destroy),
-				   G_OBJECT (w));
-
 	return w;
 }
 
@@ -654,23 +670,56 @@ panel_info_dialog (const char *class,
 		va_end (ap);
 	}
 
-	w = gtk_message_dialog_new (NULL, 0, GTK_MESSAGE_INFO,
-				    GTK_BUTTONS_OK, "foo");
-	g_signal_connect (G_OBJECT (w), "event",
-			  G_CALLBACK (panel_dialog_window_event), NULL);
-	gtk_window_set_wmclass (GTK_WINDOW (w),
-				class, "Panel");
-
-	gtk_label_set_markup (GTK_LABEL (GTK_MESSAGE_DIALOG (w)->label), s);
-
+	w = panel_dialog (NULL, GTK_MESSAGE_INFO, class, s);
 	g_free (s);
+	return w;
+}
 
-	gtk_widget_show_all (w);
+GtkWidget *
+panel_error_dialog_with_parent (GtkWindow *parent,
+				const char *class,
+				const char *format,
+				...)
+{
+	GtkWidget *w;
+	char *s;
+	va_list ap;
 
-	g_signal_connect_swapped (G_OBJECT (w), "response",
-				  G_CALLBACK (gtk_widget_destroy),
-				  G_OBJECT (w));
+	if (format == NULL) {
+		g_warning ("NULL error dialog");
+		s = g_strdup ("(null)");
+	} else {
+		va_start (ap, format);
+		s = g_strdup_vprintf (format, ap);
+		va_end (ap);
+	}
 
+	w = panel_dialog (GTK_WIDGET (parent), GTK_MESSAGE_ERROR, class, s);
+	g_free (s);
+	return w;
+}
+
+GtkWidget *
+panel_info_dialog_with_parent (GtkWindow *parent,
+			       const char *class,
+			       const char *format,
+			       ...)
+{
+	GtkWidget *w;
+	char *s;
+	va_list ap;
+
+	if (format == NULL) {
+		g_warning ("NULL info dialog");
+		s = g_strdup ("(null)");
+	} else {
+		va_start (ap, format);
+		s = g_strdup_vprintf (format, ap);
+		va_end (ap);
+	}
+
+	w = panel_dialog (GTK_WIDGET (parent), GTK_MESSAGE_INFO, class, s);
+	g_free (s);
 	return w;
 }
 
@@ -1225,7 +1274,7 @@ panel_ensure_dir (const char *dirname)
 	while (p != NULL) {
 		*p = '\0';
 		if (mkdir (parsed, 0700) != 0 &&
-		    errno != EEXIST) {
+		    errno != EEXIST && errno != ENOSYS) {
 			g_free (parsed);
 			return FALSE;
 		}
@@ -1234,7 +1283,7 @@ panel_ensure_dir (const char *dirname)
 	}
 
 	if (mkdir (parsed, 0700) != 0 &&
-	    errno != EEXIST) {
+	    errno != EEXIST && errno != ENOSYS) {
 		g_free (parsed);
 		return FALSE;
 	}
@@ -1364,8 +1413,9 @@ panel_uri_exists (const char *uri)
 
 	suri = gnome_vfs_uri_new (uri);
 
-	/* eek */
-	g_return_val_if_fail (suri != NULL, FALSE);
+	if (!suri) {
+		return FALSE;
+	}
 
 	ret = gnome_vfs_uri_exists (suri);
 
@@ -1374,81 +1424,35 @@ panel_uri_exists (const char *uri)
 	return ret;
 }
 
-#if 0
-/* hmmm, we need to use this I think to add a battery applet
- *  -George */
-/* Linux Battery Stuff */
-
-static gboolean
-linux_battery_exists (void)
+GdkPixbuf *
+missing_pixbuf (int size)
 {
-        FILE *fp;
-        char buf[200] = "";
-        int foo;
+	int i, j;
+	int rowstride;
+	guchar *pixels;
+	GdkPixbuf *pb;
 
-        if ( ! g_file_test ("/proc/apm", G_FILE_TEST_EXISTS))
-                return FALSE;
+	pb = gdk_pixbuf_new (GDK_COLORSPACE_RGB,
+			     FALSE /* has alpha */,
+			     8 /* bits per sample */,
+			     size, size);
+	pixels = gdk_pixbuf_get_pixels (pb);
+	rowstride = gdk_pixbuf_get_rowstride (pb);
 
-        fp = fopen ("/proc/apm", "r");
-        if (fp == NULL)
-                return FALSE;
+	for (i = 0; i < size; i++) {
+		for (j = 0; j < size; j++) {
+			guchar *p = pixels + i * rowstride + 3 * j;
+			if (i == j || size - i == j) {
+				p[0] = 255;
+				p[1] = 0;
+				p[2] = 0;
+			} else {
+				p[0] = 255;
+				p[1] = 255;
+				p[2] = 255;
+			}
+		}
+	}
 
-        if (fgets (buf, sizeof (buf), fp) == NULL) {
-                fclose (fp);
-                return FALSE;
-        }
-        fclose (fp);
-
-        foo = -1;
-        sscanf (buf,
-                "%*s %*d.%*d %*x %*x %*x %*x %d%% %*d %*s\n",
-                &foo);
-
-        if (foo >= 0)
-                return TRUE;
-        else
-                return FALSE;
+	return pb;
 }
-
-static gboolean
-battery_exists (void)
-{
-#ifndef __linux__
-        return FALSE;
-#else
-        /* This is MUUUUCHO ugly, but apparently RH 7.0 with segv the program
-         * reading /proc/apm on some laptops, and we don't want to crash, thus
-         * we do the check in a forked process */
-        int status;
-        pid_t pid;
-
-        pid = fork ();
-        if (pid == 0) {
-                struct sigaction sa = {{NULL}};
-
-                sa.sa_handler = SIG_DFL;
-
-                sigaction(SIGSEGV, &sa, NULL);
-                sigaction(SIGFPE, &sa, NULL);
-                sigaction(SIGBUS, &sa, NULL);
-
-                if (linux_battery_exists ())
-                        _exit (0);
-                else
-                        _exit (1);
-        }
-
-        status = 0;
-        waitpid (pid, &status, 0);
-
-        if ( ! WIFSIGNALED (status) &&
-            WIFEXITED (status) &&
-            WEXITSTATUS (status) == 0) {
-                return TRUE;
-        } else {
-                return FALSE;
-        }
-#endif
-}
-
-#endif
